import platform
import zipfile
import hashlib
import json
import os

def parseSingeRule(rule,options):
    #Parse a rule from the versions.json
    if rule["action"] == "allow":
        returnvalue = False
    elif rule["action"] == "disallow":
        returnvalue = True
    if "os" in rule:
        for key, value in rule["os"].items():
            if key == "name":
                if value == "windows" and platform.system() != 'Windows':
                    return returnvalue
                elif value == "osx" and platform.system() != 'Darwin':
                    return returnvalue
                elif value == "linux" and platform.system() != 'Linux':
                    return returnvalue
            elif key == "arch":
                if value == "x86" and platform.architecture()[0] != "32bit":
                    return returnvalue
    if "features" in rule:
        for key, value in rule["features"].items():
            if key == "has_custom_resolution" and not options.get("customResolution",False):
                return returnvalue
            elif key == "is_demo_user" and not options.get("demo",False):
                return returnvalue
    return not returnvalue

def parseRuleList(data,ruleString,options):
    #Parse a rule list
    if not ruleString in data:
        return True
    for i in data[ruleString]:
        if not parseSingeRule(i,options):
            return False
    return True


def inherit_json(original_data,path):
    #See https://github.com/tomsik68/mclauncher-api/wiki/Version-Inheritance-&-Forge
    inherit_version = original_data["inheritsFrom"]
    with open(os.path.join(path,"versions",inherit_version,inherit_version + ".json")) as f:
        new_data = json.load(f)
    for key, value in original_data.items():
        if isinstance(value,list) and isinstance(new_data.get(key,None),list):
            new_data[key] = value + new_data[key]
        elif isinstance(value,dict) and isinstance(new_data.get(key,None),dict):
            for a, b in value.items():
                if isinstance(b,list):
                    new_data[key][a] = new_data[key][a] + b
        else:
            new_data[key] = value
    return new_data

#Returns the path from a libname
def get_library_path(name,path):
    libpath = os.path.join(path,"libraries")
    base_path, libname, version = name.split(":")
    for i in base_path.split("."):
        libpath = os.path.join(libpath,i)
    try:
        version,fileend = version.split("@")
    except:
        fileend = "jar"
    libpath = os.path.join(libpath,libname,version,libname + "-" + version + "." + fileend)
    return libpath

#Returns the mainclass of a given jar
def get_jar_mainclass(path):
    zf = zipfile.ZipFile(path)
    #Parse the MANIFEST.MF
    with zf.open("META-INF/MANIFEST.MF") as f:
        lines = f.read().decode("utf-8").splitlines()
    zf.close()
    content = {}
    for i in lines:
        try:
            key, value = i.split(":")
            content[key] = value[1:]
        except:
            pass
    return content["Main-Class"]

#Returns the sha1 hash of the given file
def get_sha1_hash(path):
    BUF_SIZE = 65536
    sha1 = hashlib.sha1()
    with open(path, 'rb') as f:
        while True:
            data = f.read(BUF_SIZE)
            if not data:
                break
            sha1.update(data)
    return sha1.hexdigest()



from .install import download_file, install_minecraft_version, install_libraries
from .helper import get_library_path, get_jar_mainclass
from xml.dom import minidom
import subprocess
import platform
import requests
import tempfile
import zipfile
import random
import json
import os

def extract_file( handler, zip_path, extract_path):
    try:
        os.makedirs(os.path.dirname(extract_path))
    except:
        pass
    with handler.open(zip_path,"r") as f:
        with open(extract_path,"wb") as w:
            w.write(f.read())

def get_data_library_path(libname,path):
    #Remove the []
    libname = libname[1:-1]
    libpath = os.path.join(path,"libraries")
    base_path, libname, version, extra = libname.split(":")
    for i in base_path.split("."):
        libpath = os.path.join(libpath,i)
    try:
        extra,fileend = extra.split("@")
    except:
        fileend = "jar"
    libpath = os.path.join(libpath,libname,version,libname + "-" + version + "-" + extra + "." + fileend)
    return libpath

def forge_processors(data,path,lzma_path):
    argument_vars = {"{MINECRAFT_JAR}":os.path.join(path,"versions",data["minecraft"],data["minecraft"] + ".jar")}
    for key,value in data["data"].items():
        if value["client"].startswith("[") and value["client"].endswith("]"):
            argument_vars["{" + key + "}"] =  get_data_library_path(value["client"],path)
        else:
            argument_vars["{" + key + "}"] = value["client"]
    argument_vars["{BINPATCH}"] = lzma_path
    if platform.system() == "Windows":
        classpath_seperator = ";"
    else:
        classpath_seperator = ":"
    for i in data["processors"]:
        #Get the classpath
        classpath = ""
        for c in i["classpath"]:
            classpath = classpath + get_library_path(c,path) + classpath_seperator
        classpath = classpath + get_library_path(i["jar"],path)
        mainclass = get_jar_mainclass(get_library_path(i["jar"],path))
        command = ["java","-cp",classpath,mainclass]
        for c in i["args"]:
            var = argument_vars.get(c,c)
            if var.startswith("[") and var.endswith("]"):
                command.append(get_library_path(var[1:-1],path))
            else:
                command.append(var)
        subprocess.call(command)

def install_forge_version(versionid,path,callback=None):
    if callback == None:
        callback = {}
    FORGE_DOWNLOAD_URL = "https://files.minecraftforge.net/maven/net/minecraftforge/forge/{version}/forge-{version}-installer.jar"
    temp_file_path = os.path.join(tempfile.gettempdir(),"forge-" + str(random.randrange(1,100000)) + ".tmp")
    download_file(FORGE_DOWNLOAD_URL.format(version=versionid),temp_file_path,callback)
    zf = zipfile.ZipFile(temp_file_path,"r")
    #Read the install_profile.json
    with zf.open("install_profile.json","r") as f:
        version_content = f.read()
    version_data = json.loads(version_content)
    forge_version_id = version_data["version"]
    #Make sure, the base version is installed
    install_minecraft_version(version_data["minecraft"],path,callback=callback)
    #Install all needed libs from install_profile.json
    install_libraries(version_data,path,callback)
    #Extract the version.json
    version_json_path = os.path.join(path,"versions",forge_version_id,forge_version_id + ".json")
    extract_file(zf,"version.json",version_json_path)
    #Extract forge libs from the installer
    forge_lib_path = os.path.join(path,"libraries","net","minecraftforge","forge",versionid)
    extract_file(zf,"maven/net/minecraftforge/forge/{version}/forge-{version}.jar".format(version=versionid),os.path.join(forge_lib_path,"forge-" + versionid + ".jar"))
    extract_file(zf,"maven/net/minecraftforge/forge/{version}/forge-{version}-universal.jar".format(version=versionid),os.path.join(forge_lib_path,"forge-" + versionid + "-universal.jar"))
    #Extract the client.lzma
    lzma_path = os.path.join(tempfile.gettempdir(),"lzma-" + str(random.randrange(1,100000)) + ".tmp")
    extract_file(zf,"data/client.lzma",lzma_path)
    zf.close()
    os.remove(temp_file_path)
    #Install the rst with the vanilla function
    install_minecraft_version(forge_version_id,path,callback=callback)
    #Run the processors
    forge_processors(version_data,path,lzma_path)
    os.remove(lzma_path)

def run_forge_installer(version):
    FORGE_DOWNLOAD_URL = "https://files.minecraftforge.net/maven/net/minecraftforge/forge/{version}/forge-{version}-installer.jar"
    temp_file_path = os.path.join(tempfile.gettempdir(),"forge-" + str(random.randrange(1,100000)) + ".tmp")
    download_file(FORGE_DOWNLOAD_URL.format(version=version),temp_file_path,{})
    subprocess.call(["java","-jar",temp_file_path])
    os.remove(temp_file_path)

def list_forge_versions():
    MAVEN_METADATA_URL = "https://files.minecraftforge.net/maven/net/minecraftforge/forge/maven-metadata.xml"
    r = requests.get(MAVEN_METADATA_URL).text
    xml_data = minidom.parseString(r)
    version_list = []
    for i in xml_data.getElementsByTagName("version"):
        version_list.append(i.childNodes[0].wholeText)
    return version_list

def find_forge_version(vanilla_version):
    version_list = list_forge_versions()
    version_list.reverse()
    for i in version_list:
        version_split= i.split("-")
        if version_split[0] == vanilla_version:
            return i
    return None



import distutils.spawn
import platform
import requests
import pathlib
import json
import os

def get_minecraft_directory():
    if platform.system() == "Windows":
        return os.path.join(os.getenv('APPDATA'), ".minecraft")
    elif platform.system() == "Darwin":
        return os.path.join(str(pathlib.Path.home()), "Library", "Application Support", "minecraft")
    else:
        return os.path.join(str(pathlib.Path.home()), ".minecraft")

def get_latest_version():
    return requests.get("https://launchermeta.mojang.com/mc/game/version_manifest.json").json()["latest"]

def get_version_list():
    vlist = requests.get("https://launchermeta.mojang.com/mc/game/version_manifest.json").json()
    returnlist = []
    for i in vlist["versions"]:
        returnlist.append({"id":i["id"],"type":i["type"]})
    return returnlist

def get_installed_versions(path):
    dir_list = os.listdir(os.path.join(path,"versions"))
    version_list = []
    for i in dir_list:
        if not os.path.isdir(os.path.join(path,"versions",i)):
            continue
        with open(os.path.join(path,"versions",i,i + ".json"),"r",encoding="utf-8") as f:
            version_data = json.load(f)
        version_list.append({"id":version_data["id"],"type":version_data["type"]})
    return version_list

def get_available_versions(path):
    version_list = []
    version_check = []
    for i in get_version_list():
        version_list.append({"id":i["id"],"type":i["type"]})
        version_check.append(i["id"])
    for i in get_installed_versions(path):
        if not i["id"] in version_check:
            version_list.append(i)
    return version_list

def get_java_executable():
    if platform.system() == "Windows":
        if os.getenv("JAVA_HOME"):
            return os.path.join(os.getenv("JAVA_HOME"),"bin","java.exe")
        elif os.path.isfile("C:\Program Files (x86)\Common Files\Oracle\Java\javapath\java.exe"):
            return "C:\Program Files (x86)\Common Files\Oracle\Java\javapath\java.exe"
        else:
            return distutils.spawn.find_executable("java") or "java"
    elif os.getenv("JAVA_HOME"):
            return os.path.join(os.getenv("JAVA_HOME"),"bin","java")
    elif platform.system() == "Darwin":
        return distutils.spawn.find_executable("java") or "java"
    else:
        try:
            return os.readlink("/etc/alternatives/java")
        except:
            return distutils.spawn.find_executable("java") or "java"

def get_library_version():
    return "2.1"



#!/usr/bin/env python3
from minecraft_launcher_lib.utils import get_library_version
from setuptools import find_packages, setup

with open("README.md","r",encoding="utf-8") as f:
    description = f.read()

setup(name='minecraft-launcher-lib',
    version=get_library_version(),
    description=' A library for creating a custom Minecraft launcher',
    long_description=description,
    long_description_content_type='text/markdown',
    author='JakobDev',
    author_email='jakobdev@gmx.de',
    url='https://gitlab.com/JakobDev/minecraft-launcher-lib',
    python_requires='>=3.5',
    include_package_data=True,
    install_requires=[
        'requests',
    ],
    packages=find_packages(),
    license='BSD',
    keywords=['JakobDev','Minecraft','Mojang','launcher','minecraft-launcher','java'],
    project_urls={
        'Issue tracker': 'https://gitlab.com/JakobDev/minecraft-launcher-lib/-/issues',
        'Documentation': 'https://minecraft-launcher-lib.readthedocs.io/en/latest/index.html'
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Environment :: Other Environment',
        'License :: OSI Approved :: BSD License',
        'Topic :: Games/Entertainment',
        'Operating System :: OS Independent',
        "Operating System :: POSIX",
        "Operating System :: POSIX :: BSD",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
 )




from .helper import parseRuleList
import platform
import zipfile
import json
import os

def get_natives(data):
    #Returns the native part from the json data
    if platform.architecture()[0] == "32bit":
        arch_type = "32"
    else:
        arch_type = "64"
    if "natives" in data:
        if platform.system() == 'Windows':
            if "windows" in data["natives"]:
                return data["natives"]["windows"].replace("${arch}",arch_type)
            else:
                return ""
        elif platform.system() == 'Darwin':
            if "osx" in data["natives"]:
                return data["natives"]["osx"].replace("${arch}",arch_type)
            else:
                return ""           
        else:
            if "linux" in data["natives"]:
                return data["natives"]["linux"].replace("${arch}",arch_type)
            else:
                return "" 
    else:
        return ""

def extract_natives_file(filename,extract_path,extract_data):
    #Unpack natives
    try:
        os.mkdir(extract_path)
    except:
        pass
    zf = zipfile.ZipFile(filename,"r")
    for i in zf.namelist():
        for e in extract_data["exclude"]:
            if i.startswith(e):
                continue
        zf.extract(i,extract_path)

def extract_natives(versionid,path,extract_path):
    with open(os.path.join(path,"versions",versionid,versionid + ".json")) as f:
        data = json.load(f)
    for count, i in enumerate(data["libraries"]):
        #Check, if the rules allow this lib for the current system
        if not parseRuleList(i,"rules",{}):
            continue
        currentPath = os.path.join(path,"libraries")
        libPath, name, version = i["name"].split(":")
        for l in libPath.split("."):
            currentPath = os.path.join(currentPath,l)
        currentPath = os.path.join(currentPath,name,version)
        native = get_natives(i)
        if native == "":
            continue
        jarFilenameNative = name + "-" + version + "-" + native + ".jar"
        if "extract" in i:
            extract_natives_file(os.path.join(currentPath,jarFilenameNative),extract_path,i["extract"])
